@@ -44,7 +44,7 @@ class ThreadingTest(CoverageTest):
             fromMainThread()
             other.join()
             """,
-            [1, 3, 4, 6, 7, 9, 10, 12, 13, 14, 15], "10"
+            [1, 3, 4, 6, 7, 9, 10, 12, 13, 14, 15], "10",
         )
 
     def test_thread_run(self) -> None:
@@ -64,7 +64,7 @@ class ThreadingTest(CoverageTest):
             thd.start()
             thd.join()
             """,
-            [1, 3, 4, 5, 6, 7, 9, 10, 12, 13, 14], ""
+            [1, 3, 4, 5, 6, 7, 9, 10, 12, 13, 14], "",
         )
 
 
@@ -83,7 +83,7 @@ class RecursionTest(CoverageTest):
             recur(495)  # We can get at least this many stack frames.
             i = 8       # and this line will be traced
             """,
-            [1, 2, 3, 5, 7, 8], ""
+            [1, 2, 3, 5, 7, 8], "",
         )
 
     def test_long_recursion(self) -> None:
@@ -99,7 +99,7 @@ class RecursionTest(CoverageTest):
 
                     recur(100000)  # This is definitely too many frames.
                     """,
-                    [1, 2, 3, 5, 7], ""
+                    [1, 2, 3, 5, 7], "",
                 )
 
     def test_long_recursion_recovery(self) -> None:
@@ -112,6 +112,7 @@ class RecursionTest(CoverageTest):
         # will be traced.
 
         self.make_file("recur.py", """\
+            import sys #; sys.setrecursionlimit(70)
             def recur(n):
                 if n == 0:
                     return 0    # never hit
@@ -121,8 +122,8 @@ class RecursionTest(CoverageTest):
             try:
                 recur(100000)  # This is definitely too many frames.
             except RuntimeError:
-                i = 10
-            i = 11
+                i = 11
+            i = 12
             """)
 
         cov = coverage.Coverage()
@@ -131,21 +132,21 @@ class RecursionTest(CoverageTest):
 
         assert cov._collector is not None
         pytrace = (cov._collector.tracer_name() == "PyTracer")
-        expected_missing = [3]
+        expected_missing = [4]
         if pytrace:                                 # pragma: no metacov
-            expected_missing += [9, 10, 11]
+            expected_missing += [10, 11, 12]
 
         _, statements, missing, _ = cov.analysis("recur.py")
-        assert statements == [1, 2, 3, 5, 7, 8, 9, 10, 11]
+        assert statements == [1, 2, 3, 4, 6, 8, 9, 10, 11, 12]
         assert expected_missing == missing
 
         # Get a warning about the stackoverflow effect on the tracing function.
-        if pytrace:                                 # pragma: no metacov
+        if pytrace and not env.METACOV:                     # pragma: no metacov
             assert len(cov._warnings) == 1
             assert re.fullmatch(
                 r"Trace function changed, data is likely wrong: None != " +
                 r"<bound method PyTracer._trace of " +
-                "<PyTracer at 0x[0-9a-fA-F]+: 5 data points in 1 files>>",
+                "<PyTracer at 0x[0-9a-fA-F]+: 6 data points in 1 files>>",
                 cov._warnings[0],
             )
         else:
@@ -542,8 +543,8 @@ class GettraceTest(CoverageTest):
             """)
         status, out = self.run_command_status("python atexit_gettrace.py")
         assert status == 0
-        if env.PYPY and env.PYPYVERSION >= (5, 4):
-            # Newer PyPy clears the trace function before atexit runs.
+        if env.PYPY:
+            # PyPy clears the trace function before atexit runs.
             assert out == "None\n"
         else:
             # Other Pythons leave the trace function in place.
