@@ -9,7 +9,8 @@ import os
 import os.path
 import re
 
-from typing import Any, Dict, Iterator, Tuple, Union
+from typing import Any
+from collections.abc import Iterator
 from xml.etree import ElementTree
 
 import pytest
@@ -64,7 +65,7 @@ class XmlTestHelpers(CoverageTest):
 
     def assert_source(
         self,
-        xmldom: Union[ElementTree.Element, ElementTree.ElementTree],
+        xmldom: ElementTree.Element | ElementTree.ElementTree,
         src: str,
     ) -> None:
         """Assert that the XML has a <source> element with `src`."""
@@ -322,7 +323,7 @@ class XmlReportTest(XmlTestHelpers, CoverageTest):
     def test_no_duplicate_packages(self) -> None:
         self.make_file(
             "namespace/package/__init__.py",
-            "from . import sample; from . import test; from .subpackage import test"
+            "from . import sample; from . import test; from .subpackage import test",
         )
         self.make_file("namespace/package/sample.py", "print('package.sample')")
         self.make_file("namespace/package/test.py", "print('package.test')")
@@ -332,7 +333,7 @@ class XmlReportTest(XmlTestHelpers, CoverageTest):
         # problem occurs when they are dynamically generated during xml report
         cov = coverage.Coverage()
         with cov.collect():
-            import_local_file("foo", "namespace/package/__init__.py")
+            import_local_file("namespace.package", "namespace/package/__init__.py")
 
         cov.xml_report()
 
@@ -379,7 +380,7 @@ def unbackslash(v: Any) -> Any:
 class XmlPackageStructureTest(XmlTestHelpers, CoverageTest):
     """Tests about the package structure reported in the coverage.xml file."""
 
-    def package_and_class_tags(self, cov: Coverage) -> Iterator[Tuple[str, Dict[str, Any]]]:
+    def package_and_class_tags(self, cov: Coverage) -> Iterator[tuple[str, dict[str, Any]]]:
         """Run an XML report on `cov`, and get the package and class tags."""
         cov.xml_report()
         dom = ElementTree.parse("coverage.xml")
